package ld26;

import com.ixee.datastruct.Coordinate;
import com.ixee.datastruct.RBTree;
import com.ixee.GraphIX.GIXEngine;
import com.ixee.GraphIX.glEEngine;
import java.util.*;
import java.awt.Color;

//You know, I was a bit stumped with what I was going to do this Ludum Dare.
//
//Minimalism.
//
//Minimalism implies some interesting things.
//Minimalistic gameplay?
//	Ironic "Click this button to win!" games in which you click a button and immediately win?
//
//That's not terribly interesting, is it?
//Why don't we make something interesting for our Ludum Dare entries?
//	Hey, let's do that.
//
//Someone who isn't me, because I don't understand interesting.
//
//My interesting is not your interesting.
//
//The ResourceNode implementation of Node's interesting might be worth a try.

public abstract class Node{
	RBTree<Integer, LinkedList<Resource>> product;
	double radius;
	HashSet<Node> connections;

	boolean active = false;

	Coordinate loc;

	abstract void link(Node n);			//Implies some sort of connection between this
										//and n such that n can pull from this.
	abstract void draw(GIXEngine e);

	abstract int msToResource();

	abstract void update(long ms);

	abstract double getRadius();

	abstract double getActualRadius();

	abstract Color getColor();

	public boolean active(){
		return active;
	}
	public void toggleActive(){
		active = !active;
	}
	public Coordinate getLoc(){
		return loc;
	}

	public boolean clickedOn(Coordinate c){
		//System.out.println(loc+", "+c.pythag(loc)+" <"+radius+">");
		//System.out.println(getRadius()+", "+c.pythag(loc));
		return c.pythag(loc) < getRadius();
	}

	public boolean linked(Node n){
		//Returns true iff this node or any node this node is linked to, is linked to n.
		if(connections.isEmpty()){
			return false;
		}else{
			Iterator<Node> iter = connections.iterator();

			while(iter.hasNext()){
				Node it = iter.next();
				if(it == n || it.linked(n)){
					return true;
				}
			}
			return false;
		}
	}
	public boolean directlyLinked(Node n){
		//Returns true iff this node is linked to n.
		if(connections.isEmpty()){
			return false;
		}else{
			Iterator<Node> iter = connections.iterator();

			while(iter.hasNext()){
				Node it = iter.next();
				if(it == n){
					return true;
				}
			}
			return false;
		}
	}
	public boolean unlink(Node n){
		boolean returnable = connections.remove(n);
		if(returnable){
			n.disconnectAll();
		}
		return returnable;
	}
	public void disconnectAll(){
		for(Node n : connections){
			n.disconnectAll();
		}
		connections.clear();
	}
	public void drawLines(GIXEngine e){
		Iterator<Node> iter = connections.iterator();
		glEEngine g = e.graphics();


		float lineDepth = g.getDepth();

		float curr = lineDepth - 0.1f;
		int lineidx = 1;
		//	System.out.println(g.getDepth());

		while(iter.hasNext()){
			//draw a line from this to iter.next()
			Node end = iter.next();

		//	System.out.println(end);
		//	g.setDepth(curr+(lineidx/100f));
		//	System.out.println(g.getDepth());
			g.drawLine(loc, end.getLoc(), new Color(30, 40, 170), 2f);

			lineidx++;
		}
		//g.setDepth(lineDepth);
		//	System.out.println(g.getDepth());
		//System.out.println("line\n\n");
	}

	public Resource getResource(){
		LinkedList<Resource> maxResource = product.getMax();	//the "most important" resource
		if(maxResource == null){
			return null;
		}else{
			Resource ret = maxResource.get(0);		//if it's still in product then it has at least one element in it
			maxResource.remove(ret);
			if(maxResource.size() == 0){
				product.remove(ret.priority());
			}
			return ret;
		}
	}
	public void pull(){
		Iterator<Node> iter = connections.iterator();

		while(iter.hasNext()){
			Node it = iter.next();
			Resource r = it.getResource();
			if(r != null){
				//System.out.println(this+" got "+r);

				if(product.get(r.priority()) == null){
					product.insert(r.priority(), new LinkedList<Resource>());
				}
				product.get(r.priority()).add(r);
			}
			it.pull();
		}
	}
}