package com.ixee.GraphIX;

import org.lwjgl.opengl.Display;

import com.ixee.ui.ogl.*;
import com.ixee.ui.KeyboardInfo;
import com.ixee.ui.CursorInfo;
import java.awt.image.BufferedImage;
import com.ixee.datastruct.Coordinate;

public class GIXEngine extends Thread{	//Figure this out. I have a feeling this might be more complex than it seems.
						//Does each call in the UI menu have to be synchronized? Hm.
						//Possibly set it up such that the menu is checked, updated, and then the checkable values
						//Are only updated after this is all done?
							//Could introduce extra latenency.
							//Overall, it would run faster though, so the extra latency would be lessened.

	glEEngine graphics;
	CursorInfo cursorInfo;
	KeyboardInfo keyboardInfo;
	MenuStackOGL menus;
	Coordinate mouseLoc = new Coordinate(0,0,0);
	Coordinate clickLoc = null;

	boolean intercepted = false;

	Coordinate topLeft, bottomLeft, bottomRight, topRight;
	Coordinate top, bottom, left, right;

	public GIXEngine(int x, int y){
		this("Default Name", x, y);
	}
	public GIXEngine(String s, int x, int y){
		this(s, x, y, new String[]{});
	}
	public GIXEngine(String s, int x, int y, String[] order){
		graphics = new glEEngine(s, order, x, y);
		graphics.setMode(glEEngine.game);
		GIXInputWrapper.initInput();
		menus = new MenuStackOGL();
		cursorInfo = new CursorInfo(graphics.getSize().mult(0.5), GIXVariables.mouseButtons.length);
		keyboardInfo = new KeyboardInfo(GIXVariables.keyPresses.length);

		topLeft		=	new Coordinate(-x/2,  y/2);
		bottomLeft	=	new Coordinate(-x/2, -y/2);
		bottomRight	=	new Coordinate( x/2, -y/2);
		topRight	=	new Coordinate( x/2,  y/2);

		left 		=	new Coordinate(-x/2,  0);
		right		=	new Coordinate( x/2,  0);
		top			=	new Coordinate( 0,  y/2);
		bottom		=	new Coordinate( 0, -y/2);

	}
	public void addMenu(MenuOGL menu){
		menus.addMenu(menu);
	}
	public void removeMenu(MenuOGL menu){
		menus.removeMenu(menu);
	}
	public void renderMenus(){
		/*
			graphics draw stuff
		*/
		menus.draw(graphics);
	}
	public MenuStackOGL menus(){
		return menus;
	}
	public void setRenderOrder(String[] renderOrder){
		graphics.setRenderOrder(renderOrder);
	}
	public glEImage loadImage(BufferedImage b){
		return graphics.loadImage(b);
	}
	public void update(){
		graphics.update();
		GIXInputWrapper.poll();
		cursorInfo.sync(GIXVariables.mousePosition.sub(graphics.getSize().mult(0.5)), GIXVariables.mouseButtons, GIXVariables.mouseButtonsReleased);
		keyboardInfo.sync(GIXVariables.keyPresses);
		//System.out.println(cursorInfo.position());
		mouseLoc = cursorInfo.position();
		checkMouse();
		graphics.clear();
	}
	public boolean mouseIntercepted(){
		return mouseLoc==null;
	}
	public Coordinate mousePosition(){
		return mouseLoc;
	}
	public Coordinate clickLoc(){
		//Returns the click location of the last update, if any, or null otherwise.
		//Careful careful.
		return clickLoc;
	}
	public void checkMouse(){
		clickLoc = null;
		//boolean intercepted = false;
		intercepted = menus.check(cursorInfo, keyboardInfo);
		if(!intercepted && cursorInfo.clicked(0)){
			clickLoc = cursorInfo.releaseLoc(0);
			//set up returnable
		}
	}
	public glEEngine graphics(){
		return graphics;
	}
	public KeyboardInfo getKeyboardState(){
		return keyboardInfo;
	}
	public CursorInfo getCursorState(){
		return cursorInfo;
	}
	public Coordinate getSize(){
		return graphics.getSize();
	}
	public float cos(float d){
		return GIXVariables.fastMath.cos(d);
	}
	public float sin(float d){
		return GIXVariables.fastMath.cos(d);
	}
	public boolean closeRequested(){
		return graphics.closeRequested();
	}
	public Coordinate topLeft(){
		return topLeft;
	}
	public Coordinate bottomLeft(){
		return bottomLeft;
	}
	public Coordinate bottomRight(){
		return bottomRight;
	}
	public Coordinate topRight(){
		return topRight;
	}
	public Coordinate left(){
		return left;
	}
	public Coordinate right(){
		return right;
	}
	public Coordinate top(){
		return top;
	}
	public Coordinate bottom(){
		return bottom;
	}
	public void resize(double x, double y){
		topLeft		=	new Coordinate(-x/2,  y/2);
		bottomLeft	=	new Coordinate(-x/2, -y/2);
		bottomRight	=	new Coordinate( x/2, -y/2);
		topRight	=	new Coordinate( x/2,  y/2);

		left 		=	new Coordinate(-x/2,  0);
		right		=	new Coordinate( x/2,  0);
		top			=	new Coordinate( 0,  y/2);
		bottom		=	new Coordinate( 0, -y/2);

		menus.resize();
	}
	public boolean clickIntercepted(){
		return intercepted;
	}
	public boolean pressed(char c){
		return GIXVariables.keyPresses[c-'a'] && GIXVariables.diffs[c-'a'];
	}
	public int windowX(){
		return Display.getX();
	}
	public int windowY(){
		return Display.getY();
	}
	public int windowWidth(){
		return Display.getWidth();
	}
	public int windowHeight(){
		return Display.getHeight();
	}
}