package com.ixee.GraphIX;

import com.ixee.datastruct.Coordinate;

public class glECamera{
	Coordinate view;
	double rotation;
	glEMatrix projection;
	double depth;

	public glECamera(Coordinate cam, Coordinate v, double spin){
		projection = new glEMatrix();
		translate(cam.getX(), cam.getY());
		rotate(spin);

		//depth = (float)cam.getZ();

		view = v;
	}
	public Coordinate getLocation(){
		return new Coordinate(projection.getMatrix()[6], projection.getMatrix()[7], depth);
	}
	public Coordinate getViewBounds(){
		//This is the height and width of what the camera can see.
		return view;
	}
	public void setDepth(double f){
		depth = f;
	}
	public double getDepth(){
		return depth;
	}
	public void setViewBounds(Coordinate c){
		//Sets the width/height of the camera view as it would be for z = 1.
		view = c;
	}
	public glECamera translate(double x, double y){
		projection.translate(x, y);
		return this;
	}
	public glECamera translate_p(double x, double y){
		projection.translate_p(x, y);
		return this;
	}
	public glECamera scale(double x, double y){
		projection.scale(x, y);
		return this;
	}
	public glECamera scale_p(double x, double y){
		projection.scale_p(x, y);
		return this;
	}
	public glECamera rotate(double d){
		rotation = d;
		projection.rotate(d);
		return this;
	}
	public glECamera rotate_p(double d){
		rotation = d;
		projection.rotate_p(d);
		return this;
	}
	public void pushProjection(){
		projection.push();
	}
	public void popProjection(){
		projection.pop();
	}
	public double[] project(double[] point){
		return projection.project(point);
	}
	public double[] batchProject(double[] points, int stride){
		return projection.batchProject(points, stride);
	}
	public boolean visible(double[] point){
		double[] result = project(point);
		return ( Math.abs(point[0]) < view.getX()/2 ) && ( Math.abs(point[1]) < view.getY()/2 );
	}
	public void load(double[] matrix){
		projection.load(matrix);
	}
	public void identity(){
		projection.load(projection.identity());
	}
	public void invert(){
		//Loads this camera with the inverse of its current projection matrix.
		double[] curr = projection.getMatrix();
		double[] inverse = new double[9];
		double det = curr[0]*(curr[4]*curr[8]-curr[5]*curr[7]) -
					curr[1]*(curr[8]*curr[3]-curr[5]*curr[6]) +
					curr[2]*(curr[3]*curr[7]-curr[4]*curr[6]);

		inverse[0] = curr[4]*curr[8]-curr[5]*curr[7];
		inverse[1] = curr[5]*curr[6]-curr[8]*curr[3];
		inverse[2] = curr[3]*curr[7]-curr[4]*curr[6];
		inverse[3] = curr[2]*curr[7]-curr[1]*curr[8];
		inverse[4] = curr[0]*curr[8]-curr[2]*curr[6];
		inverse[5] = curr[6]*curr[1]-curr[0]*curr[7];
		inverse[6] = curr[1]*curr[5]-curr[2]*curr[4];
		inverse[7] = curr[2]*curr[3]-curr[0]*curr[5];
		inverse[8] = curr[0]*curr[4]-curr[1]*curr[3];

		for(int i=0;i<8;i++){		//Potential bug. Why not all nine?
			inverse[i] /= det;
		}

		projection.load(inverse);
	}
}