/*Todo: See if changing this class to have hashmaps to the values (or rather, one hash map to the indices
		of the values may be better) instead of straight up slowly searching arrays.
*/

package com.ixee.GraphIX;

import com.ixee.datastruct.Coordinate;
import com.ixee.io.stdio;
import java.awt.image.*;
import java.util.*;
import java.awt.Color;

public class glESprite{
	//HashMap<String, Integer> indiceMap;
	float[][] drawInfo;
	Coordinate[] sizes;
	BufferedImage[] images;
	String[] names;
	Integer[] indices;
	boolean[] visible;
	Color[] blendColor;
	String typeName;
	public double x=0f;
	public double y=0f;
	public float spin=0f;
	boolean shadable = true;
	glEVBOSpriteData[] vboData;

	public glESprite(BufferedImage[] source, String[] sourceNames, String sourceName){
		//indiceMap = new HashMap<String, Integer>();

		images = source;
		names = sourceNames;
		typeName = sourceName;
		vboData = null;
		drawInfo = new float[source.length][7]; //{alpha cutoff zoom, alpha rate, alpha default, size cutoff zoom, size rate, size default}
		for(int i=0;i<drawInfo.length;i++){
			drawInfo[i][0] = 0;
			drawInfo[i][1] = 0;
			drawInfo[i][2] = 1;
			drawInfo[i][3] = 0;
			drawInfo[i][4] = 0;
			drawInfo[i][5] = 1;
			drawInfo[i][6] = 1;
		}
		visible = new boolean[source.length];
		for(int i=0;i<visible.length;i++){
			visible[i] = true;
		}
		blendColor = new Color[source.length];
		for(int i=0;i<blendColor.length;i++){
			blendColor[i] = new Color(1f,1f,1f,1f);
		}
		sizes = new Coordinate[source.length];
		for(int i=0;i<sizes.length;i++){
			sizes[i] = new Coordinate(source[i].getWidth(null), source[i].getHeight(null));
		}
	}
	public glESprite(BufferedImage[] source, String[] sourceNames, float[][] drawInfos, String sourceName){
		images = source;
		names = sourceNames;
		typeName = sourceName;
		vboData = null;
		drawInfo = drawInfos;
		visible = new boolean[source.length];
		for(int i=0;i<visible.length;i++){
			visible[i] = true;
		}
		blendColor = new Color[source.length];
		for(int i=0;i<blendColor.length;i++){
			blendColor[i] = new Color(1f,1f,1f,1f);
		}
	}
	public void initVBOData(glEVBOSpriteData[] data){
		vboData = data;
	}
	public boolean vboEstablished(){
		return (vboData != null);
	}
	public glEVBOSpriteData[] getVBOData(){
		return vboData;
	}
	public void setDrawInfo(float[][] infos){
		if(images.length == infos.length){
			boolean valid = true;
			for(int i=0;i<infos.length;i++){
				if(infos[i].length != 7){
					valid = false;
					stdio.debugAndLog(this, "Inalid drawInfo array. Length of array for \""+typeName+"("+names[i]+")\" is not 6");
				}
			}
			if(valid){
				drawInfo = infos;
			}
		}else{
			stdio.debugAndLog(this, "Invalid drawInfo array. Length does not match number of images in this sprite. ("+typeName+")");
		}
	}
	public void setDrawInfo(String name, float[] info){
		int index = -1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		if(index != -1 && info.length == 7){
			drawInfo[index] = info;
		}else{
		//	stdio.debug(this, "Invalid sprite name. Resource not found.");
		}
	}
	public Integer[] getSpriteIndices(){
		return Arrays.copyOf(indices, indices.length);
	}
	public boolean shadable(){
		return shadable;
	}
	public void setShadable(boolean b){
		shadable = b;
	}
	public void setVisibility(String name, boolean visibility){
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		if(index != -1){
			visible[index] = visibility;
		}else{
		//	stdio.debug(this, "Resource not found in sprite: "+name);
		}
	}
	public boolean isVisible(String name){
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		if(index != -1){
			return visible[index];
		}else{
			return false;
		}
	}
	public void setColor(String name, Color col){
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		//make a getIndex method to get the index of sprite with the name name.
		if(index != -1){
			blendColor[index] = col;
		}
	}
	public Color getColor(String name){
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		return blendColor[index];
	}
	public BufferedImage getImage(String name){
		BufferedImage returnable = null;
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		if(index == -1){
			stdio.debugAndLog(this, "Sprite requested was not found in this sprite. "+name);
		}else{
			returnable = images[index];
		}
		return returnable;
	}
	public float[][] getDrawInfo(){
		return drawInfo;
	}
	public float[] getDrawInfo(String name){
		float[] returnable = null;
		for(int i=0;i<names.length;i++){
			if(names[i].equals(name)){
				returnable = drawInfo[i];
			}
		}
		return returnable;
	}
	public BufferedImage[] getImages(){
		return images;
	}
	public String[] getResources(){
		return names;
	}
	public String getTypeName(){
		return typeName;
	}
	public void setPosition(Coordinate c){
		x = (float)c.getX();
		y = (float)c.getY();
	}
	public void setSpin(Coordinate c){
		spin = (float)c.getX();
	}
	public Integer getSpriteIndex(String name){
		Integer returnable = null;
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		if(index != -1){
		//	System.out.println("DURRR DE DURR "+indices +" " + index + ", " + typeName);
			returnable = indices[index];
		}else{
		//	stdio.debug(this, "Sprite requested was not found in this object ("+name+")");
		}
		return returnable;
	}
	public void setSpriteIndices(Integer[] sprites){
		indices = sprites;
	}
	public int numResources(){
		return visible.length;
	}
	public Coordinate getSize(String name){
		Coordinate returnable = null;
		int index=-1;
		for(int i=0;i<names.length;i++){
			if(names[i].equalsIgnoreCase(name)){
				index = i;
			}
		}
		if(index != -1){
			returnable = sizes[index];
		}
		return returnable;
	}
}