//defines a coordinate, angular position, velocity, angular velocity, or
//any other triplet of values.
//
//contains a Double[] where [0] is x, [1] is y, and [2] is z
//
//+x is to the left, +y is up, and +z is forward
//
//for rotations, x = yz plane, y = xz plane, and z = xy plane
// for x-rotation, 0 is +z axis
// for y-rotation, 0 is +z axis
// for z-rotation, 0 is +x axis

package com.ixee.datastruct;

public class Coordinate{
	double px, py, pz;

	public static void main(String[] args){
		Coordinate c = new Coordinate(5, -5, 3);

		long start;

		start = System.nanoTime();

		for(int i=0;i<10000000;i++){
			c.mult_p(1.0/7000.0);

			c.add_p(1.0, 2.0, 3.5);

			c.mult_p(2);
			c.mult_p(1/3.0);


			c.mult_p(3.0);
			c.mult_p(1.0/2.0);

			c.sub_p(1.0, 2.0, 3.5);

			c.mult_p(7000);

		}
		System.out.println("Done in " + (System.nanoTime()-start)/1000000 + "ms");


		start = System.nanoTime();

		for(int i=0;i<10000000;i++){
			c = c.mult(1.0/7000.0);

			c = c.add(1.0, 2.0, 3.5);

			c = c.mult(2);
			c = c.mult(1/3.0);


			c = c.mult(3.0);
			c = c.mult(1.0/2.0);

			c = c.sub(1.0, 2.0, 3.5);

			c = c.mult(7000);

		}
		System.out.println("Done in " + (System.nanoTime()-start)/1000000 + "ms");

		System.out.println(c);

	}


	public Coordinate(double x, double y){
		px = x;
		py = y;
		pz = 0;
	}
	public Coordinate(double x, double y, double z){
		px = x;
		py = y;
		pz = z;
	}
	public Coordinate(Coordinate c){
		px = c.getX();
		py = c.getY();
		pz = c.getZ();
	}
	public Coordinate(double[] d) throws ArrayIsWrongSizeException{
		if(d.length == 3){
			px = d[0];
			py = d[1];
			pz = d[2];
		}else{
			throw new ArrayIsWrongSizeException(3, d.length);
		}
	}
	public double getX(){
		return px;
	}
	public double getY(){
		return py;
	}
	public double getZ(){
		return pz;
	}
	public int intX(){
		return (int)Math.round(px);
	}
	public int intY(){
		return (int)Math.round(py);
	}
	public int intZ(){
		return (int)Math.round(pz);
	}
	public double[] getCoords(){
		return new double[]{px, py, pz};
	}
	public double getCoords(int i) throws IndexOutOfBoundsException{
		switch(i){
			case 0:
				return px;

			case 1:
				return py;

			case 2:
				return pz;

			default:
				throw new IndexOutOfBoundsException();
		}
	}
	public int getInt(int i){
		int returnable = 0;
		try{
			returnable = (int)Math.round(getCoords(i));
		}catch(IndexOutOfBoundsException ioobe){
			ioobe.printStackTrace();
		}
		return returnable;
	}
	public void setX(double x){
		px=x;
	}
	public void setY(double y){
		py=y;
	}
	public void setZ(double z){
		pz=z;
	}
	public void setCoords(double[] d) throws ArrayIsWrongSizeException{
		if(d.length!=3){
			throw new ArrayIsWrongSizeException(3, d.length);
		}else{
			px = d[0];
			py = d[1];
			pz = d[2];
		}
	}
	public void setCoords(int i, double d) throws IndexOutOfBoundsException{
		switch(i){
			case 0:
				px = d;
				break;

			case 1:
				py = d;
				break;

			case 2:
				pz = d;
				break;

			default:
				throw new IndexOutOfBoundsException();
		}
	}
	public void setCoords(Coordinate c){
		px = c.getX();
		py = c.getY();
		pz = c.getZ();
	}
	public void setCoords(double x, double y){
		px = x;
		py = y;
		pz = 0;
	}
	public void setCoords(double x, double y, double z){
		px = x;
		py = y;
		pz = z;
	}
	public boolean isCloseEnough(Coordinate c, double threshold){
		boolean closeEnough = true;
		for(int i=0;i<3;i++){
			if(Math.abs(this.getCoords(i)-c.getCoords(i))>threshold){
				closeEnough = false;
			}
		}
		return closeEnough;
	}
	public Coordinate add_p(Coordinate c){
		px += c.getX();
		py += c.getY();
		pz += c.getZ();
		return this;
	}
	public Coordinate add_p(double x, double y){
		px += x;
		py += y;
		return this;
	}
	public Coordinate add_p(double x, double y, double z){
		px += x;
		py += y;
		pz += z;
		return this;
	}
	public Coordinate sub_p(Coordinate c){
		px -= c.getX();
		py -= c.getY();
		pz -= c.getZ();
		return this;
	}
	public Coordinate sub_p(double x, double y){
		px -= x;
		py -= y;
		return this;
	}
	public Coordinate sub_p(double x, double y, double z){
		px -= x;
		py -= y;
		pz -= z;
		return this;
	}
	public Coordinate mult_p(double d){
		px *= d;
		py *= d;
		pz *= d;
		return this;
	}

	public Coordinate add(Coordinate c){
		return new Coordinate(this).add_p(c);
	}
	public Coordinate add(double x, double y, double z){
		return new Coordinate(this).add_p(x, y, z);
	}
	public Coordinate add(double x, double y){
		return new Coordinate(this).add_p(x, y);
	}
	public Coordinate sub(Coordinate c){
		return new Coordinate(this).sub_p(c);
	}
	public Coordinate sub(double x, double y, double z){
		return new Coordinate(this).sub_p(x, y, z);
	}
	public Coordinate sub(double x, double y){
		return new Coordinate(this).sub_p(x, y);
	}
	public Coordinate mult(double d){
		return new Coordinate(this).mult_p(d);
	}
	public String toString(){
		StringBuilder b = new StringBuilder();
		b.append(getX());
		b.append(" ");
		b.append(getY());
		b.append(" ");
		b.append(getZ());
		return b.toString();
	}
	public double pythag(){
		return pythag(0,0,0);
	}
	public double pythag(Coordinate c){
		return pythag(c.getX(), c.getY(), c.getZ());
	}
	public double pythag(double x, double y, double z){
		return Math.sqrt(sqrpythag(x, y, z));
	}
	public double pythag(double x, double y){
		return pythag(x, y, 0);
	}
	public double sqrpythag(double x, double y, double z){
		return (px-x)*(px-x)+(py-y)*(py-y)+(pz-z)*(pz-z);
	}
	public double sqrpythag(double x, double y){
		return sqrpythag(x, y, 0);
	}
	public double sqrpythag(){
		return sqrpythag(0, 0, 0);
	}
	public double sqrpythag(Coordinate c){
		return pythag(c.getX(), c.getY(), c.getZ());
	}
}