import com.ixee.GraphIX.*;
import com.ixee.io.*;
import java.io.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

public class AnimPacker{
	public static void main(String[] args){
		String[] instrBuffer = args;
		System.out.println("Started with "+args.length+" arguments. Closing on end");
		boolean closeOnEnd = false;
		if(instrBuffer.length>0){
			closeOnEnd = true;
		}
		Input in = new Input();
		String[] options = {"path", "pack", "unpack", "exit", "list"};
		String path = ".";
		boolean running = true;
		int state;
		int instrCounter = 0;
		while(running){
			state = -1;
			if(instrCounter<instrBuffer.length){
				for(int i=0;i<options.length;i++){
					if(options[i].equalsIgnoreCase(instrBuffer[instrCounter])){
						state = i;
					}
				}
				System.out.println("Doing instr "+instrBuffer[instrCounter]);
				instrCounter++;
			}else{
				if(closeOnEnd){
					state = 3;
				}else{
					state = getInput(in, options, "Instruction");
				}
			}
			switch(state){
				case 0:
					String newPath;
					if(instrCounter < instrBuffer.length){
						newPath = instrBuffer[instrCounter];
						instrCounter++;
					}else{
						newPath = in.getInput("Please enter new path");
					}
					if(newPath.equals("")){
						System.out.println(path);
					}else{
						File f;
						if(newPath.length()>1 && newPath.charAt(1)==':'){
							f = new File(newPath);
						}else{
							f = new File(path.concat("/"+newPath));
						}
						if(f.exists()){
							System.out.println("Setting path to "+f.getAbsolutePath());
							path = f.getAbsolutePath();
						}else{
							System.out.println(f.getAbsolutePath() +" does not exist!");
						}
					}
					break;
				case 1:
					String toPack;
					if(instrCounter < instrBuffer.length){
						toPack = instrBuffer[instrCounter];
						instrCounter++;
					}else{
						toPack = in.getInput("What are we packing?");
					}
					pack(path, toPack);
					break;
				case 2:
					String toUnpack;
					if(instrCounter < instrBuffer.length){
						toUnpack = instrBuffer[instrCounter];
						instrCounter++;
					}else{
						toUnpack = in.getInput("What are we unpacking?");
					}
					unpack(path, toUnpack);
					break;
				case 3:
					running = false;
					break;
				case 4:
					String[] contents = new File(path).list();
					System.out.println("List of directories in "+path+":");
					for(int i=0;i<contents.length;i++){
						if(new File(path.concat("/"+contents[i])).isDirectory()){
							System.out.println(contents[i]);
						}
					}
					break;
			}
		}
	}
	public static void pack(String path, String toPack){
		File filesDir = new File(path.concat("/"+toPack));
		try{
			System.out.println("Finding "+filesDir.getCanonicalPath());
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
		if(filesDir.exists()){
			String[] fileList = filesDir.list();
			ArrayList<String> images = new ArrayList<String>();
			String archiveSettings = null;
			for(int i=0;i<fileList.length;i++){
				String tempPath = fileList[i];
				if(new File(path.concat("/"+toPack+"/"+tempPath)).isFile()){
					System.out.println("Checking "+path.concat("/"+toPack+"/"+tempPath));
					String extension = tempPath.substring(tempPath.lastIndexOf("."), tempPath.length());
					if(extension.equalsIgnoreCase(".png") || extension.equalsIgnoreCase(".bmp")){
						images.add(fileList[i]);
					}else if(extension.equalsIgnoreCase(".txt")){
						archiveSettings = fileList[i];
					}
				}else{
					System.out.println("Ignoring a folder: "+path.concat("/"+toPack+"/"+tempPath));
				}
			}
			glEAnimArchive outputArchive = new glEAnimArchive(images.size());
			for(int i=0;i<images.size();i++){
				String tempPath = images.get(i);
				System.out.println("Loading file "+path.concat("/"+toPack+"/"+tempPath));
				int imageIndex = Integer.parseInt(tempPath.substring(tempPath.lastIndexOf(".")-1, tempPath.lastIndexOf(".")));
				outputArchive.setImage(imageIndex, ImageLoader.loadFile(path.concat("/"+toPack+"/"+tempPath)));
			}
			if(outputArchive.getImages().length>0){
				outputArchive.setName(toPack);
				if(archiveSettings != null){
					String[] settings = TextLoader.loadFile(archiveSettings);
					outputArchive.setMsPerFrame(Integer.parseInt(settings[0]));
					outputArchive.setDefaultLoops(Integer.parseInt(settings[1]));
				}else{
					outputArchive.setMsPerFrame(1/outputArchive.getImages().length);
					outputArchive.setDefaultLoops(1);
				}
				try{
					File output = new File(path.concat("/"+toPack).concat(".gixani"));
					System.out.println("Writing "+output.getAbsolutePath());
					output.createNewFile();
					FileOutputStream fos = new FileOutputStream(output);
					fos.write(outputArchive.asBytes());
					fos.close();
				}catch(IOException ioe){
					ioe.printStackTrace();
				}
			}else{
				System.out.println("No images found, packing aborted");
			}
		}else{
			System.out.println("Folder does not exist");
		}
	}
	public static void unpack(String path, String toUnpack){
		glEAnimArchive animArchive = new glEAnimArchive(path.concat(toUnpack).concat(".gixani"));
		BufferedImage[] toWrite = animArchive.getImages();
		String[] settingsOut = new String[2];
		settingsOut[0] = Integer.toString(animArchive.getMsPerFrame());
		settingsOut[1] = Integer.toString(animArchive.getDefaultLoops());

		for(int i=0;i<toWrite.length;i++){
			ImageWriter.writeImage(toWrite[i], animArchive.name().concat(Integer.toString(i)).concat(".png"));
		}
		TextWriter.writeFile(settingsOut, path.concat(toUnpack).concat(".txt"));
	}
	public static int getInput(Input in, String[] options, String whatToDo){
		int returnable = 0;
		String input = in.getInput(whatToDo);
		returnable = in.checkValidity(input, options);
		while(returnable == -1){
			input = in.getInput(whatToDo);
			returnable = in.checkValidity(input, options);
		}
		return returnable;
	}
}