package com.ixee.ui;

import java.util.LinkedList;
import com.ixee.io.stdio;

public class KeyboardInfo{

	class Keypress{
		boolean[] modifiers = new boolean[6];
		public static final int CONTROL_L = 0;
		public static final int CONTROL_R = 1;
		public static final int ALT_L = 2;
		public static final int ALT_R = 3;
		public static final int SHIFT_L = 4;
		public static final int SHIFT_R = 5;
		int key;
		public Keypress(int k, boolean lctrl, boolean rctrl, boolean llt, boolean rlt, boolean lsft, boolean rsft){
			key = k;
			modifiers[CONTROL_L] = lctrl;
			modifiers[CONTROL_R] = rctrl;
			modifiers[ALT_L] = llt;
			modifiers[ALT_R] = rlt;
			modifiers[SHIFT_L] = lsft;
			modifiers[SHIFT_R] = rsft;
		}
		public int getKey(){
			return key;
		}
		public boolean isAlphaNumeric(){
			return false;
		}
		public boolean isModifiedBy(int flag){
			return modifiers[flag];
		}
	}

	public static final int LSHIFT = 26;
	public static final int RSHIFT = 27;
	public static final int LCONTROL = 28;
	public static final int RCONTROL = 29;
	public static final int ESCAPE = 30;
	public static final int RETURN = 31;
	public static final int PAUSE = 32;
	public static final int BACKSPACE = 33;
	public static final int SPACE = 34;
	public static final int LALT = 35;
	public static final int RALT = 36;
	public static final int LBRACE = 47;
	public static final int RBRACE = 48;
	public static final int MINUS = 49;
	public static final int EQUALS = 50;
	public static final int BSLASH = 51;
	public static final int FSLASH = 52;
	public static final int SEMICOLON = 53;
	public static final int APOSTROPHE = 54;
	public static final int COMMA = 55;
	public static final int PERIOD = 56;
	public static final int GRAVE = 57;
	public static final int DELETE = 58;
	public static final int TAB = 60;

	LinkedList<Keypress> bufferedPresses = new LinkedList<Keypress>();

	boolean[] pressedKeys;
	boolean[] alteredKeys;
	/*
		Allow for a raw state dump of they keyboard, or deal with the keyboard on a key-by-key basis with the Keypress event system.
	*/
	public KeyboardInfo(int numKeys){
		pressedKeys = new boolean[numKeys];
		alteredKeys = new boolean[numKeys];
	}
	public void sync(boolean[] pressStates){
		boolean cap = false;
		for(int i=0;i<pressStates.length;i++){
			if(pressStates[i] != pressedKeys[i]){
				alteredKeys[i] = true;
				pressedKeys[i] = pressStates[i];
				if(pressedKeys[i]){
					cap = true;
					bufferedPresses.add(new Keypress(i, controlL(), controlR(), altL(), altR(), shiftL(), shiftR()));
				}
			}
		}
		if(cap){
			bufferedPresses.add(new Keypress(-1, false, false, false, false, false, false));
		}
	}
	public boolean hasNext(){
		return bufferedPresses.size() > 1;
	}
	public int numEventsBuffered(){
		return bufferedPresses.size();
	}
	public boolean isKeyPressed(char key){
		int toWorkWith = key;
		return pressedKeys[key];
	}
	public boolean isKeyUp(char key){
		return !isKeyPressed(key);
	}
	public int keyPressed(){
		int returnable = 0;
		try{
			returnable = bufferedPresses.peekFirst().getKey();
		}catch(NullPointerException npe){
		//	stdio.debugAndLog(this, "No key events to read!");
			returnable = -1;
		}
		return returnable;
	}
	public boolean altL(){
		return pressedKeys[LALT];
	}
	public boolean altR(){
		return pressedKeys[RALT];
	}
	public boolean shiftL(){
		return pressedKeys[LSHIFT];
	}
	public boolean shiftR(){
		return pressedKeys[RSHIFT];
	}
	public boolean controlL(){
		return pressedKeys[LCONTROL];
	}
	public boolean controlR(){
		return pressedKeys[RCONTROL];
	}
	public boolean keyShift(){
		boolean returnable = false;
		try{
			Keypress temp = bufferedPresses.peekFirst();
			returnable = temp.isModifiedBy(Keypress.SHIFT_L) | temp.isModifiedBy(Keypress.SHIFT_R);
		}catch(NullPointerException npe){
			stdio.debugAndLog(this, "No key events to read!");
			returnable = false;
		}
		return returnable;
	}
	public boolean keyAlt(){
		boolean returnable = false;
		try{
			Keypress temp = bufferedPresses.peekFirst();
			returnable = temp.isModifiedBy(Keypress.ALT_L) | temp.isModifiedBy(Keypress.ALT_R);
		}catch(NullPointerException npe){
			stdio.debugAndLog(this, "No key events to read!");
			returnable = false;
		}
		return returnable;
	}
	public boolean keyControl(){
		boolean returnable = false;
		try{
			Keypress temp = bufferedPresses.peekFirst();
			returnable = temp.isModifiedBy(Keypress.CONTROL_L) | temp.isModifiedBy(Keypress.CONTROL_R);
		}catch(NullPointerException npe){
			stdio.debugAndLog(this, "No key events to read!");
			returnable = false;
		}
		return returnable;
	}
	public boolean nextKey(){
		boolean returnable;
		if(bufferedPresses.size() > 0){
			returnable = true;
			bufferedPresses.pollFirst();
		}else{
			returnable = false;
		}
		return returnable;
	}
	public char keyAsChar(){
		char toReturn = 0;
		int keyIndex = keyPressed();
		String returnable = "";
		if(keyIndex < 0){
			returnable = "";
		}else if(keyIndex < 26){
			keyIndex += 65;
			if(!keyShift()){
				keyIndex = keyIndex | 32;
			}
			returnable = new Character((char)keyIndex).toString();
		}else if(keyIndex > 36 && keyIndex < 47){
			if(!keyShift()){
				returnable = new Character((char)(keyIndex+11)).toString();
			}else{
				if(keyIndex == 46){
					returnable = "!";
				}else if(keyIndex == 38){
					returnable = "@";
				}else if(keyIndex == 39){
					returnable = "#";
				}else if(keyIndex == 40){
					returnable = "$";
				}else if(keyIndex == 41){
					returnable = "%";
				}else if(keyIndex == 42){
					returnable = "^";
				}else if(keyIndex == 43){
					returnable = "&";
				}else if(keyIndex == 44){
					returnable = "*";
				}else if(keyIndex == 45){
					returnable = "(";
				}else if(keyIndex == 37){
					returnable = ")";
				}
			}
		}else if(keyIndex == LBRACE){
			if(keyShift()){
				returnable = "{";
			}else{
				returnable = "[";
			}
		}else if(keyIndex == RBRACE){
			if(keyShift()){
				returnable = "}";
			}else{
				returnable = "]";
			}
		}else if(keyIndex == MINUS){
			if(keyShift()){
				returnable = "_";
			}else{
				returnable = "-";
			}
		}else if(keyIndex == EQUALS){
			if(keyShift()){
				returnable = "+";
			}else{
				returnable = "=";
			}
		}else if(keyIndex == BSLASH){
			if(keyShift()){
				returnable = "|";
			}else{
				returnable = "\\";
			}
		}else if(keyIndex == FSLASH){
			if(keyShift()){
				returnable = "?";
			}else{
				returnable = "/";
			}
		}else if(keyIndex == SEMICOLON){
			if(keyShift()){
				returnable = ":";
			}else{
				returnable = ";";
			}
		}else if(keyIndex == APOSTROPHE){
			if(keyShift()){
				returnable = "\"";
			}else{
				returnable = "'";
			}
		}else if(keyIndex == COMMA){
			if(keyShift()){
				returnable = "<";
			}else{
				returnable = ",";
			}
		}else if(keyIndex == PERIOD){
			if(keyShift()){
				returnable = ">";
			}else{
				returnable = ".";
			}
		}else if(keyIndex == GRAVE){
			if(keyShift()){
				returnable = "~";
			}else{
				returnable = "`";
			}
		}else if(keyIndex == RETURN){
			toReturn = (char)10;
		}else if(keyIndex == BACKSPACE){
			System.out.println("Backspace.");
			toReturn = (char)8;
		}else if(keyIndex == DELETE){
			toReturn = (char)127;
		}else if(keyIndex == TAB){
			toReturn = (char)9;
		}if(keyIndex == 34){
			return ' ';
		}

		if(returnable.length() != 0){
			toReturn = returnable.charAt(0);
		}
		return toReturn;
	}
}