package com.ixee.ui;

import java.util.ArrayList;

//all of the menus override doStuff(), which is the mechanism whereby
//menus actually get things done.

//Menus simply extend the UI, which takes already-defined parts of the game
//and presents them and their relations in more understandable and human-readable
//ways.

//All of the components that the UI uses should have pre-structured and placed,
//because of the aforementioned postulate.

public class Menu implements AbstractMenu{
	protected ArrayList<UIElement> elements;
	protected ArrayList<UIActiveElement> activeElements;
	protected boolean top;
	protected boolean uncounting;
	protected double closeTime;
	protected double closeStartTime;
	protected MenuStack parent;
	int mode = -1;


	public Menu(MenuStack m){
		parent = m;
		elements = new ArrayList<UIElement>();
		activeElements = new ArrayList<UIActiveElement>();
		top = false;
		uncounting = false;
		closeTime = 0;
		closeStartTime = 0;
	}
	public ArrayList<UIElement> getElements(){
		return elements;
	}
	public boolean topMenu(){
		return top;
	}
	public void doStuff(){
		//we do stuff here. Standard menu doesn't.
		//the idea is to use a switch()case statement to decide though.
	}
	public String whatWillThisDo(int i){
		//return a string stating what this method will do for
		//for the given mode.
		return "This is the default menu. It won't do anything for any mode.";
	}
	public int validModes(){
		//returns the number of valid modes this menu has.
		//thus, the valid modes themselves range from 0 to returnable-1
		return activeElements.size();
	}
	public void draw(){
		for(int i=0;i<elements.size();i++){
			elements.get(i).draw();
		}
	}
	public void check(CursorInfo listen){
		for(int i=0;i<activeElements.size();i++){
			activeElements.get(i).testForActive(listen);
			if(activeElements.get(i).isActive()){
				mode = activeElements.get(i).getMode();
			}
		}
		doStuff();
	}
	public double getCloseDelay(){
		double returnable = 0;
		for(int i=0;i<elements.size();i++){
			returnable = Math.max(elements.get(i).getCloseDelay(), returnable);
		}
		return returnable;
	}
	public boolean closed(){
		return (getCloseDelay() > (System.currentTimeMillis()-closeStartTime));
	}
	public void close(){
		closeStartTime = System.currentTimeMillis();
		for(int i=0;i<elements.size();i++){
			elements.get(i).close();
		}
	}
}