package com.ixee.ui;

import com.ixee.datastruct.Coordinate;

public abstract class UIElement{

	public static final int H_LEFT = 0x01;
	public static final int H_RIGHT = 0x02;
	public static final int H_CENTER = 0x03;
	public static final int V_TOP = 0x10;
	public static final int V_BOTTOM = 0x20;
	public static final int V_CENTER = 0x30;
	public static final int B_LEFT = (byte)(H_LEFT|V_BOTTOM);
	public static final int B_RIGHT = (byte)(H_RIGHT|V_BOTTOM);
	public static final int T_LEFT = (byte)(H_LEFT|V_TOP);
	public static final int T_RIGHT = (byte)(H_RIGHT|V_TOP);

	protected Coordinate centerpoint;
	protected double closeDelay;
	public UIElement(Coordinate c){
		centerpoint = c;
		closeDelay = 0;
	}
	public boolean contains(Coordinate c){
		return false;
	}
	public void setCloseDelay(double d){
		closeDelay = d;
	}
	public abstract void close();
		//do whatever we need to "close" this item.
		//in the case of most things, the close delay is 0 and close() does nothing.
	public double getCloseDelay(){
		return closeDelay;
	}
	public abstract double top();
	public abstract double bottom();
	public abstract double left();
	public abstract double right();
	public Coordinate topLeft(){
		return new Coordinate(left(), top());
	}
	public Coordinate topRight(){
		return new Coordinate(right(), top());
	}
	public Coordinate bottomLeft(){
		return new Coordinate(left(), bottom());
	}
	public Coordinate bottomRight(){
		return new Coordinate(right(), bottom());
	}
	public Coordinate topCoord(){
		return new Coordinate(centerpoint.getX(), top());
	}
	public Coordinate leftCoord(){
		return new Coordinate(left(), centerpoint.getY());
	}
	public Coordinate bottomCoord(){
		return new Coordinate(centerpoint.getX(), bottom());
	}
	public Coordinate rightCoord(){
		return new Coordinate(right(), centerpoint.getY());
	}
}