package com.ixee.ui.ogl;

import java.awt.Color;
import com.ixee.ui.CursorInfo;
import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.glEEngine;

public class UIButton extends UIActiveElement{
	protected Coordinate bounds;
	protected String text = "";
	protected String activeText = "";
	protected Color backColor = Color.BLUE;
	protected Color activeColor = Color.RED;
	protected Color textColor = new Color(20, 250, 40);
	protected boolean pressed = false;
	double[] boundingPoly = new double[8];
	Color focusColor = new Color(120, 50, 100);
	double downscale = 0.9;

	public UIButton(Coordinate position, Coordinate b, int mode, String s, Color back, Color active){
		super(position, mode);
		bounds = b;
		text = s;
		activeText = s;
		backColor = back;
		activeColor = active;
		boundingPoly[0] = b.getX()/2;
		boundingPoly[1] = b.getY()/2;
		boundingPoly[2] = b.getX()/2;
		boundingPoly[3] = -b.getY()/2;
		boundingPoly[4] = -b.getX()/2;
		boundingPoly[5] = -b.getY()/2;
		boundingPoly[6] = -b.getX()/2;
		boundingPoly[7] = b.getY()/2;
		for(int i=0;i<boundingPoly.length;i+=2){
			boundingPoly[i] = boundingPoly[i]*downscale + position.getX();
			boundingPoly[i+1] = boundingPoly[i+1]*downscale + position.getY();
		}
	}
	public UIButton(Coordinate position, Coordinate b, int mode, String s){
		this(position, b, mode, s, Color.BLUE, Color.RED);
	}
	public UIButton(Coordinate position, Coordinate b, int mode){
		this(position, b, mode, "");
	}
	public UIButton(Coordinate position, Coordinate b){
		this(position, b, -1);
	}
	/*
		Make SetAppearance and setActiveAppearance take color and/or coordinate valies to define their locations and sizes.
	*/
	public Coordinate getBounds(){
		return bounds;
	}
	public void setAppearance(){}
	public void setAppearance(Color c){
		backColor = c;
	}
	public Color getColor(){
		return backColor;
	}
	public void setActiveAppearance(){}
	public void setActiveAppearance(Color c){
		activeColor = c;
	}
	public Color getActiveColor(){
		return activeColor;
	}
	public void setActiveText(String s){
		activeText = s;
	}
	public void setTextColor(Color c){
		textColor = c;
	}
	public void setText(String s){
		text = s;
	}
	public void rotate(){
		double x = bounds.getX();
		bounds.setX(bounds.getY());
		bounds.setY(x);
	}
	public void testForActive(CursorInfo listen){
		Coordinate testPoint = listen.releaseLoc(CursorInfo.LEFT);
		pressed = false;
		if(listen.pressed(CursorInfo.LEFT) && contains(listen.pressLoc(CursorInfo.LEFT))){
			pressed = true;
		}
		if(listen.clicked(CursorInfo.LEFT) && contains(testPoint)){
			toggleActiveMode = true;
		}
		makeActive();
		//grab cursor info
		//if the cursor is in the bounds of this item,
		//and mouse <appropriate number> is pressed
		//set this to be active
	}
	public void toggleColors(){
		Color c = backColor;
		backColor = activeColor;
		activeColor = c;
	}
	public boolean contains(Coordinate c){
		boolean returnable = false;
		double leftBound = centerpoint.getX()-bounds.getX()/2;
		double rightBound = centerpoint.getX()+bounds.getX()/2;
		double lowerBound = centerpoint.getY()-bounds.getY()/2;
		double upperBound = centerpoint.getY()+bounds.getY()/2;
		//System.out.print("Checking for "+c+" in " + leftBound + " "+rightBound+" "+lowerBound+" "+upperBound+"...");
		if(c.getX() > leftBound && c.getX() < rightBound){
			if(c.getY() > lowerBound && c.getY() < upperBound){
				returnable = true;
			}
		}
		//System.out.println(" " + returnable);
		return returnable;
	}
	public void draw(glEEngine e){
		if(pressed){
			e.fillRect(centerpoint.add(new Coordinate(bounds.getX()/2, bounds.getY()/2, 0)), centerpoint.sub(new Coordinate(bounds.getX()/2, bounds.getY()/2, 0)), activeColor);
			e.drawString(activeText, centerpoint.getX(), centerpoint.getY(), textColor);
		}else{
			e.fillRect(centerpoint.add(new Coordinate(bounds.getX()/2, -bounds.getY()/2, 0)), centerpoint.sub(new Coordinate(bounds.getX()/2, -bounds.getY()/2, 0)), backColor);
			e.drawString(text, centerpoint.getX(), centerpoint.getY(), textColor);
		}
		if(focused){
			e.drawPoly(boundingPoly, focusColor);
		}
	}
	public double right(){
		return centerpoint.getX()+bounds.getX()/2;
	}
	public double left(){
		return centerpoint.getX()-bounds.getX()/2;
	}
	public double top(){
		return centerpoint.getY()+bounds.getY()/2;
	}
	public double bottom(){
		return centerpoint.getY()-bounds.getY()/2;
	}
	public void close(){}
}