//TODO: Handle resizing.

/*
	ScrollBar simply is a generic |<|-------[]--|>| type element.
	Takes a slider and adds two buttons to it to increment in some directions.
*/

package com.ixee.ui.ogl;

import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.*;
import com.ixee.ui.CursorInfo;

public class UIScrollBar extends UIActiveElement{
	UIButton inc;
	UIButton dec;
	UISlider slider;
	Coordinate bounds;

	private final static int INC_PRESSED = 1;
	private final static int DEC_PRESSED = 2;
	private final static int SLIDER_PRESSED = 0;

	public UIScrollBar(Coordinate c, Coordinate b, int mode){
		super(c, mode);
		bounds = b;
		double thickness = b.getX();
		double length = b.getY();
		if(b.getY()<b.getX()){
			thickness = b.getY();
			length = b.getX();
		}
		double buttonLength = Math.min(10*Math.sqrt(Math.max(thickness, 10)/10), 25);
		double buttonX = length/2-buttonLength;

		inc = new UIButton(new Coordinate(buttonX+buttonLength/2, 0).add(c), new Coordinate(buttonLength, thickness), INC_PRESSED);
		dec = new UIButton(new Coordinate(-buttonX-buttonLength/2, 0).add(c), new Coordinate(buttonLength, thickness), DEC_PRESSED);
		slider = new UISlider(c, new Coordinate(buttonX*2, thickness), SLIDER_PRESSED);

		if(b.getY() > b.getX()){
			rotate();
		}

	}
	public double getValue(){
		return slider.getValue();
	}
	private void rotate(){
		Coordinate oldCenter = inc.getCenter().sub(centerpoint);
		Coordinate oldBounds;

		inc.setCenter(new Coordinate(oldCenter.getY(), oldCenter.getX()).add(centerpoint));
		inc.rotate();

		oldCenter = dec.getCenter().sub(centerpoint);
		dec.setCenter(new Coordinate(oldCenter.getY(), oldCenter.getX()).add(centerpoint));
		dec.rotate();

		oldBounds = slider.getBounds();
		slider.setBounds(new Coordinate(oldBounds.getY(), oldBounds.getX()));
		bounds = new Coordinate(bounds.getY(), bounds.getX());
	}
	public double getRatio(){
		return slider.getRatio();
	}
	public void draw(glEEngine e){
		slider.draw(e);
		inc.draw(e);
		dec.draw(e);
	}
	public void resize(double d){
		slider.resize(d);
	}
	public void setIncrementAmount(double d){
		slider.setIncrementAmount(d);
	}
	public void testForActive(CursorInfo inf){
		//This is only set as active if its value is changed in some manner.
		Coordinate pressLoc;
		int thingPressed = -1;
		if(inf.pressed(0) || inf.clicked(0)){
			pressLoc = inf.pressLoc(0).sub(centerpoint);
			inc.testForActive(inf);
			dec.testForActive(inf);
			slider.testForActive(inf);

			thingPressed = Math.max(thingPressed, inc.getMode());
			thingPressed = Math.max(thingPressed, dec.getMode());
			thingPressed = Math.max(thingPressed, slider.getMode());


			if(thingPressed != -1){
				toggleActiveMode = true;

				switch(thingPressed){
					case INC_PRESSED:
						System.out.println("Incrementing");
						slider.increment();
						break;
					case DEC_PRESSED:
						System.out.println("Decrementing");
						slider.decrement();
						break;
				//	case SLIDER_PRESSED:
				//		System.out.println("Jumping");
				//		if(slider.isVertical()){
				//			slider.setBarValue(pressLoc.getY()+slider.getLength()+dec.getBounds().getY());
				//		}else{
				//			slider.setBarValue(pressLoc.getX()+slider.getLength()+dec.getBounds().getX());
				//		}
				//		break;
				}
			}
		}
		makeActive(); //toggles this to be active until getMode() is called, at which point this is set as non-active and returns its mode/
				//Activity is to denote what was interacted with on this particular tick.
	}
	public void setActiveAppearance(){}
	public void setAppearance(){}
	public double right(){
		return centerpoint.getX()+bounds.getX()/2;
	}
	public double left(){
		return centerpoint.getX()-bounds.getX()/2;
	}
	public double top(){
		return centerpoint.getY()+bounds.getY()/2;
	}
	public double bottom(){
		return centerpoint.getY()-bounds.getY()/2;
	}
	public void close(){}
}