/*
	TODO: Add support for vertical justification of text.
*/

package com.ixee.ui.ogl;

import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.glEEngine;
import java.util.*;
import com.ixee.font.FontDataAccelerated;
import java.awt.*;

public class UITextBox extends UIElement{
	double width;
	double height;

	byte alignment;

	boolean textChanged;
	boolean drawBoxEdges;

	String textsrc;
	String splitText;

	ArrayList<Integer> textData;



	public UITextBox(Coordinate c, double w, double h){
		this(c, w, h, "");
	}
	public UITextBox(Coordinate c, double w, double h, String s){
		super(c);
		width = w;
		height = h;
		textData = new ArrayList<Integer>();
		textsrc = s;
		drawBoxEdges = false;	//Don't draw box outline by default.
		setText(textsrc);
	}

	public void setAlignment(byte b){
		alignment = b;
	}

	public void drawBoxEdges(boolean b){
		drawBoxEdges = b;
	}

	public void draw(glEEngine e){
		//Vertical justification does nothing (yet)

		int leftness = alignment & 0x0f;
		int topness =  alignment & 0xf0;

		int leftOffset;

		int yiter;
		if(textChanged){
			setText(textsrc, e);
			textChanged = false;
		}

		FontDataAccelerated font = e.getFont();

		if(topness == UIElement.V_TOP){
			yiter = (int)Math.round((height-font.getHeight())/2);
		}else if(topness == UIElement.V_BOTTOM){
			yiter = (int)Math.round((-height/2+(textData.size()-1)*font.getHeight()));
		}else{
			yiter = (int)Math.round((textData.size()+1)*font.getHeight()/2);
		}

		for(int i=0;i<textData.size()-1 && -yiter < height/2;i++){
			if(yiter <= height/2){
				if(leftness == UIElement.H_LEFT){
					leftOffset = (int)Math.round(-(width-font.getWidth(textsrc, textData.get(i), textData.get(i+1)-1))/2);
				}else if(leftness == UIElement.H_RIGHT){
					leftOffset = (int)Math.round((width-font.getWidth(textsrc, textData.get(i), textData.get(i+1)-1))/2);
				}else{
					leftOffset = 0;
				}
				e.drawString(textsrc, textData.get(i), textData.get(i+1)-1, centerpoint.getX() + leftOffset, centerpoint.getY()+yiter, new Color(255, 255, 255));
			}
			yiter-=font.getHeight();
		}
		if(drawBoxEdges){
			//draw some edges!
		}
		//e.drawString(textsrc, textData.get(textData.size()-1)
		//draw it, blah blah
	}
	public void setText(String s){
		textChanged = true;
		textsrc = s;
	}
	public void setText(String s, glEEngine e){
		FontDataAccelerated font = e.getFont();
		if(font == null){
			throw new NullPointerException("No font is loaded! Cannot draw text.");
		}
		textData.clear();
		textData.add(0);

		int cursorpos = 0;
		int lastSpace = -1;
		int currlinelength = 0;

		while(cursorpos < s.length()){
			//seek forward
			if(s.charAt(cursorpos) == ' '){
				//add the word's length to the line length
				int wordlength = font.getWidth(s, textData.get(textData.size()-1), cursorpos);
				if(wordlength >= width || s.charAt(cursorpos) == '\n'){
					currlinelength = 0;
					textData.add(lastSpace+1);
				}
				currlinelength += font.getWidth(s, lastSpace, cursorpos);
				lastSpace = cursorpos;
			}
			cursorpos++;
		}

		int wordlength = font.getWidth(s, textData.get(textData.size()-1), s.length());
		if(wordlength >= width){
			currlinelength = 0;
			textData.add(lastSpace+1);
		}
		currlinelength += font.getWidth(s, lastSpace, cursorpos);
		textData.add(s.length()+1);
	}

	public void setWidth(int width){
		this.width = width;
		setText(textsrc);
	}

	public void close(){}
	public double right(){
		return centerpoint.getX()+width/2;
	}
	public double left(){
		return centerpoint.getX()-width/2;
	}
	public double top(){
		return centerpoint.getY()+height/2;
	}
	public double bottom(){
		return centerpoint.getY()-height/2;
	}
}